@@ -12,18 +12,62 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import contextlib
 import logging
-import bunch
+
 from django.conf import settings
-from horizon.exceptions import ServiceCatalogException
+from django.contrib.messages import api as msg_api
+from django.utils.encoding import force_unicode
+from django.utils.translation import ugettext_lazy as _
+from horizon import exceptions
 from openstack_dashboard.api.base import url_for
-from muranoclient.v1.client import Client
-from muranodashboard.dynamic_ui.services import get_service_name
+import muranoclient.client as client
 from muranoclient.common.exceptions import HTTPForbidden, HTTPNotFound
 from consts import STATUS_ID_READY, STATUS_ID_NEW
-from .network import get_network_params
+from muranoclient.common import exceptions as exc
+from muranodashboard.environments import topology
+from muranodashboard.common import utils
 
-log = logging.getLogger(__name__)
+
+LOG = logging.getLogger(__name__)
+
+
+def _handle_message(request, message):
+    def horizon_message_already_queued(_message):
+        _message = force_unicode(_message)
+        if request.is_ajax():
+            for tag, msg, extra in request.horizon['async_messages']:
+                if _message == msg:
+                    return True
+        else:
+            for msg in msg_api.get_messages(request):
+                if msg.message == _message:
+                    return True
+        return False
+
+    if horizon_message_already_queued(message):
+        exceptions.handle(request, ignore=True)
+    else:
+        exceptions.handle(request, message=message)
+
+
+@contextlib.contextmanager
+def handled_exceptions(request):
+    """Handles all murano-api specific exceptions."""
+    try:
+        yield
+    except exc.CommunicationError:
+        msg = _('Unable to communicate to murano-api server.')
+        LOG.exception(msg)
+        _handle_message(request, msg)
+    except exc.Unauthorized:
+        msg = _('Check Keystone configuration of murano-api server.')
+        LOG.exception(msg)
+        _handle_message(request, msg)
+    except exc.Forbidden:
+        msg = _('Operation is forbidden by murano-api server.')
+        LOG.exception(msg)
+        _handle_message(request, msg)
 
 
 def get_endpoint(request):
@@ -33,9 +77,9 @@ def get_endpoint(request):
     if not endpoint:
         try:
             endpoint = url_for(request, 'murano')
-        except ServiceCatalogException:
+        except exceptions.ServiceCatalogException:
             endpoint = 'http://localhost:8082'
-            log.warning('Murano API location could not be found in Service '
+            LOG.warning('Murano API location could not be found in Service '
                         'Catalog, using default: {0}'.format(endpoint))
     return endpoint
 
@@ -45,16 +89,17 @@ def muranoclient(request):
     insecure = getattr(settings, 'MURANO_API_INSECURE', False)
 
     token_id = request.user.token.id
-    log.debug('Murano::Client <Url: {0}, '
+    LOG.debug('Murano::Client <Url: {0}, '
               'TokenId: {1}>'.format(endpoint, token_id))
 
-    return Client(endpoint=endpoint, token=token_id, insecure=insecure)
+    return client.Client(
+        1, endpoint=endpoint, token=token_id, insecure=insecure)
 
 
 def get_status_messages_for_service(request, service_id, environment_id):
     deployments = muranoclient(request).deployments. \
         list(environment_id)
-    log.debug('Deployment::List {0}'.format(deployments))
+    LOG.debug('Deployment::List {0}'.format(deployments))
 
     result = '\n'
     #TODO: Add updated time to logs
@@ -122,21 +167,21 @@ class Session(object):
                     muranoclient(request).sessions.get(environment_id, id)
             except HTTPForbidden:
                 del sessions[environment_id]
-                log.debug("The environment is deploying by other user."
+                LOG.debug("The environment is deploying by other user."
                           "Creating a new session "
                           "for the environment {0}".format(environment_id))
                 return create_session(request, environment_id)
             else:
                 if session_data.state == "deployed":
                     del sessions[environment_id]
-                    log.debug("The existent session has status 'deployed'."
+                    LOG.debug("The existent session has status 'deployed'."
                               " Creating a new session "
                               "for the environment {0}".format(environment_id))
                     return create_session(request, environment_id)
         else:
-            log.debug("Creating a new session")
+            LOG.debug("Creating a new session")
             return create_session(request, environment_id)
-        log.debug("Found active session "
+        LOG.debug("Found active session "
                   "for the environment {0}".format(environment_id))
         return id
 
@@ -155,17 +200,19 @@ class Session(object):
         session_id = sessions[environment_id] \
             if environment_id in sessions else None
         if session_id:
-            log.debug("Using session_id {0} for the environment {1}".format(
+            LOG.debug("Using session_id {0} for the environment {1}".format(
                 session_id, environment_id))
         else:
-            log.debug("Session for the environment {0} not found".format(
+            LOG.debug("Session for the environment {0} not found".format(
                 environment_id))
         return session_id
 
 
 def environments_list(request):
-    environments = muranoclient(request).environments.list()
-    log.debug('Environment::List {0}'.format(environments))
+    environments = []
+    with handled_exceptions(request):
+        environments = muranoclient(request).environments.list()
+    LOG.debug('Environment::List {0}'.format(environments))
     for index, env in enumerate(environments):
         environments[index].has_services = False
         environment = environment_get(request, env.id)
@@ -180,33 +227,32 @@ def environments_list(request):
 
 
 def environment_create(request, parameters):
-    body = get_network_params(request)
     #name is required param
-    body['name'] = parameters['name']
+    body = {'name': parameters['name']}
     env = muranoclient(request).environments.create(body)
-    log.debug('Environment::Create {0}'.format(env))
+    LOG.debug('Environment::Create {0}'.format(env))
     return env
 
 
 def environment_delete(request, environment_id):
-    log.debug('Environment::Delete <Id: {0}>'.format(environment_id))
+    LOG.debug('Environment::Delete <Id: {0}>'.format(environment_id))
     return muranoclient(request).environments.delete(environment_id)
 
 
 def environment_get(request, environment_id):
     session_id = Session.get(request, environment_id)
-    log.debug('Environment::Get <Id: {0}, SessionId: {1}>'.
+    LOG.debug('Environment::Get <Id: {0}, SessionId: {1}>'.
               format(environment_id, session_id))
     env = muranoclient(request).environments.get(environment_id, session_id)
-    log.debug('Environment::Get {0}'.format(env))
+    LOG.debug('Environment::Get {0}'.format(env))
     return env
 
 
 def environment_deploy(request, environment_id):
     session_id = Session.get(request, environment_id)
-    log.debug('Session::Get <Id: {0}>'.format(session_id))
+    LOG.debug('Session::Get <Id: {0}>'.format(session_id))
     env = muranoclient(request).sessions.deploy(environment_id, session_id)
-    log.debug('Environment::Deploy <EnvId: {0}, SessionId: {1}>'
+    LOG.debug('Environment::Deploy <EnvId: {0}, SessionId: {1}>'
               ''.format(environment_id, session_id))
     return env
 
@@ -233,16 +279,18 @@ def services_list(request, environment_id):
 
     for service_item in environment.services:
         service_data = service_item
-        service_data['full_service_name'] = get_service_name(
-            request, service_data['type'])
+        service_id = service_data['?']['id']
 
-        if service_data['id'] in reports and reports[service_data['id']]:
-            last_operation = strip(str(reports[service_data['id']].text))
-            time = reports[service_data['id']].updated.replace('T', ' ')
+        if service_id in reports and reports[service_id]:
+            last_operation = strip(str(reports[service_id].text))
+            time = reports[service_id].updated.replace('T', ' ')
         else:
-            last_operation = 'Service draft created' \
+            last_operation = 'Component draft created' \
                 if environment.version == 0 else ''
-            time = service_data['updated'].replace('T', ' ')[:-7]
+            try:
+                time = service_data['updated'].replace('T', ' ')[:-7]
+            except KeyError:
+                time = None
 
         service_data['environment_id'] = environment_id
         service_data['environment_version'] = environment.version
@@ -250,22 +298,26 @@ def services_list(request, environment_id):
         service_data['operation_updated'] = time
         services.append(service_data)
 
-    log.debug('Service::List')
-    return [bunch.bunchify(service) for service in services]
+    LOG.debug('Service::List')
+    return [utils.Bunch(**service) for service in services]
 
 
-def service_list_by_type(request, environment_id, service_type):
+def app_id_by_fqn(request, fqn):
+    apps = muranoclient(request).packages.filter(fqn=fqn)
+    return apps[0].id if apps else None
+
+
+def service_list_by_fqn(request, environment_id, fqn):
     services = services_list(request, environment_id)
-    log.debug('Service::Instances::List')
-    return [service for service in services
-            if service['type'] == service_type]
+    LOG.debug('Service::Instances::List')
+    return [service for service in services if service['?']['type'] == fqn]
 
 
 def service_create(request, environment_id, parameters):
     # we should be able to delete session
     # if we what add new services to this environment
     session_id = Session.get_or_create_or_delete(request, environment_id)
-    log.debug('Service::Create {0}'.format(parameters['type']))
+    LOG.debug('Service::Create {0}'.format(parameters['?']['type']))
     return muranoclient(request).services.post(environment_id,
                                                path='/',
                                                data=parameters,
@@ -273,7 +325,7 @@ def service_create(request, environment_id, parameters):
 
 
 def service_delete(request, environment_id, service_id):
-    log.debug('Service::Delete <SrvId: {0}>'.format(service_id))
+    LOG.debug('Service::Delete <SrvId: {0}>'.format(service_id))
     session_id = Session.get_or_create_or_delete(request, environment_id)
     return muranoclient(request).services.delete(environment_id,
                                                  '/' + service_id,
@@ -282,14 +334,14 @@ def service_delete(request, environment_id, service_id):
 
 def service_get(request, environment_id, service_id):
     services = services_list(request, environment_id)
-    log.debug("Return service detail for a specified id")
+    LOG.debug("Return service detail for a specified id")
     for service in services:
-        if service.id == service_id:
+        if service['?']['id'] == service_id:
             return service
 
 
 def deployments_list(request, environment_id):
-    log.debug('Deployments::List')
+    LOG.debug('Deployments::List')
     deployments = muranoclient(request).deployments.list(environment_id)
     for deployment in deployments:
         if deployment.started:
@@ -297,21 +349,21 @@ def deployments_list(request, environment_id):
         if deployment.finished:
             deployment.finished = deployment.finished.replace('T', ' ')
 
-    log.debug('Environment::List {0}'.format(deployments))
+    LOG.debug('Environment::List {0}'.format(deployments))
     return deployments
 
 
 def deployment_reports(request, environment_id, deployment_id):
-    log.debug('Deployment::Reports::List')
+    LOG.debug('Deployment::Reports::List')
     reports = muranoclient(request).deployments.reports(environment_id,
                                                         deployment_id)
-    log.debug('Deployment::Reports::List {0}'.format(reports))
+    LOG.debug('Deployment::Reports::List {0}'.format(reports))
     return reports
 
 
 def get_deployment_start(request, environment_id, deployment_id):
     deployments = muranoclient(request).deployments.list(environment_id)
-    log.debug('Get deployment start time')
+    LOG.debug('Get deployment start time')
     for deployment in deployments:
         if deployment.id == deployment_id:
             return deployment.started.replace('T', ' ')
@@ -320,13 +372,13 @@ def get_deployment_start(request, environment_id, deployment_id):
 
 def get_deployment_descr(request, environment_id, deployment_id):
     deployments = muranoclient(request).deployments.list(environment_id)
-    log.debug('Get deployment description')
+    LOG.debug('Get deployment description')
     for deployment in deployments:
         if deployment.id == deployment_id:
-            descr = deployment.description
-            if 'services' in descr:
-                for service in descr['services']:
-                    service['full_service_name'] = get_service_name(
-                        request, service['type'])
-            return descr
+            return deployment.description
     return None
+
+
+def load_environment_data(request, environment_id):
+    environment = environment_get(request, environment_id)
+    return topology.render_d3_data(environment)
