@@ -12,7 +12,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from django.core.urlresolvers import reverse, reverse_lazy
+from django.core.urlresolvers import reverse
 from django.utils.translation import ugettext_lazy as _
 from django import shortcuts
 
@@ -21,6 +21,7 @@ from horizon import tables
 from horizon import messages
 
 from muranodashboard.environments import api
+from muranodashboard.environments import consts
 from muranodashboard.openstack.common import timeutils
 from consts import STATUS_ID_DEPLOYING
 from consts import STATUS_CHOICES
@@ -29,17 +30,29 @@ from consts import STATUS_ID_NEW
 from consts import DEPLOYMENT_STATUS_DISPLAY_CHOICES
 
 
-class CreateService(tables.LinkAction):
-    name = 'CreateService'
-    verbose_name = _('Create Service')
-    classes = ('btn-launch', 'ajax-modal')
+def creation_allowed(self, request, environment):
+    environment_id = self.table.kwargs['environment_id']
+    env = api.environment_get(request, environment_id)
+    status = getattr(env, 'status', None)
+
+    if status not in [STATUS_ID_DEPLOYING]:
+        return True
+    return False
+
+
+class AddApplication(tables.LinkAction):
+    name = 'AddApplication'
+    verbose_name = _('Add Component')
+    classes = ('btn-launch',)
 
     def allowed(self, request, environment):
-        environment_id = self.table.kwargs['environment_id']
-        status, = api.get_environment_data(request, environment_id, 'status')
-        if status not in [STATUS_ID_DEPLOYING]:
-            return True
-        return False
+        return creation_allowed(self, request, environment)
+
+    def get_link_url(self, datum=None):
+        base_url = reverse('horizon:murano:catalog:switch_env',
+                           args=(self.table.kwargs['environment_id'],))
+        redirect_url = reverse('horizon:murano:catalog:index')
+        return '{0}?next={1}'.format(base_url, redirect_url)
 
 
 class CreateEnvironment(tables.LinkAction):
@@ -89,12 +102,13 @@ class EditEnvironment(tables.LinkAction):
 
 
 class DeleteService(tables.DeleteAction):
-    data_type_singular = _('Service')
-    data_type_plural = _('Services')
+    data_type_singular = _('Component')
+    data_type_plural = _('Components')
 
     def allowed(self, request, service=None):
         environment_id = self.table.kwargs.get('environment_id')
-        status, = api.get_environment_data(request, environment_id, 'status')
+        env = api.environment_get(request, environment_id)
+        status = getattr(env, 'status', None)
 
         return False if status == STATUS_ID_DEPLOYING else True
 
@@ -102,7 +116,7 @@ class DeleteService(tables.DeleteAction):
         try:
             environment_id = self.table.kwargs.get('environment_id')
             for service in self.table.data:
-                if service.id == service_id:
+                if service['?']['id'] == service_id:
                     api.service_delete(request,
                                        environment_id,
                                        service_id)
@@ -145,8 +159,10 @@ class DeployThisEnvironment(tables.Action):
 
     def allowed(self, request, service):
         environment_id = self.table.kwargs['environment_id']
-        status, version = api.get_environment_data(request, environment_id,
-                                                   'status', 'version')
+        env = api.environment_get(request, environment_id)
+        status = getattr(env, 'status', None)
+        version = getattr(env, 'version', None)
+
         if status == STATUS_ID_DEPLOYING:
             return False
         services = self.table.data
@@ -171,7 +187,7 @@ class DeployThisEnvironment(tables.Action):
 
 class ShowEnvironmentServices(tables.LinkAction):
     name = 'show'
-    verbose_name = _('Services')
+    verbose_name = _('Components')
     url = 'horizon:murano:environments:services'
 
     def allowed(self, request, environment):
@@ -225,7 +241,11 @@ class EnvironmentsTable(tables.DataTable):
 
 def get_service_details_link(service):
     return reverse('horizon:murano:environments:service_details',
-                   args=(service.environment_id, service.id))
+                   args=(service.environment_id, service['?']['id']))
+
+
+def get_service_type(datum):
+    return datum['?'].get(consts.DASHBOARD_ATTRS_KEY, {}).get('name')
 
 
 class ServicesTable(tables.DataTable):
@@ -233,41 +253,29 @@ class ServicesTable(tables.DataTable):
                          verbose_name=_('Name'),
                          link=get_service_details_link)
 
-    _type = tables.Column('full_service_name',
+    _type = tables.Column(get_service_type,
                           verbose_name=_('Type'))
 
-    status = tables.Column('status',
+    status = tables.Column(lambda datum: datum['?'].get('status'),
                            verbose_name=_('Status'),
                            status=True,
                            status_choices=STATUS_CHOICES,
                            display_choices=STATUS_DISPLAY_CHOICES)
-
     operation = tables.Column('operation',
                               verbose_name=_('Last operation'))
-
     operation_updated = tables.Column('operation_updated',
                                       verbose_name=_('Time updated'))
 
-    def __init__(self, request, **kwargs):
-        # we need to update CreateService url here because by default actions
-        # are instantiated in DataTableMetaclass with no args
-        for action_id, action in self.base_actions.items():
-            if isinstance(action, CreateService):
-                self.base_actions[action_id] = CreateService(
-                    url=reverse('horizon:murano:environments:create',
-                                args=(kwargs.get('environment_id'),)))
-                break
-        super(ServicesTable, self).__init__(request, **kwargs)
-
     def get_object_id(self, datum):
-        return datum.id
+        return datum['?']['id']
 
     class Meta:
         name = 'services'
-        verbose_name = _('Services')
+        verbose_name = _('Components')
         status_columns = ['status']
         row_class = UpdateServiceRow
-        table_actions = (CreateService, DeleteService, DeployThisEnvironment)
+        table_actions = (AddApplication, DeleteService,
+                         DeployThisEnvironment)
         row_actions = (DeleteService,)
 
 
@@ -305,12 +313,13 @@ class DeploymentsTable(tables.DataTable):
 class EnvConfigTable(tables.DataTable):
     name = tables.Column('name',
                          verbose_name=_('Name'))
-    _type = tables.Column('full_service_name',
-                          verbose_name=_('Type'))
+    _type = tables.Column(
+        lambda datum: datum['?'][consts.DASHBOARD_ATTRS_KEY]['name'],
+        verbose_name=_('Type'))
 
     def get_object_id(self, datum):
-        return datum['id']
+        return datum['?']['id']
 
     class Meta:
         name = 'environment_configuration'
-        verbose_name = _('Deployed Services')
+        verbose_name = _('Deployed Components')
